@@ -7,6 +7,11 @@
  */
 
 
+#ifdef WIN32
+#include "win32config.h"
+#else
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include "xmlmemory.h"
@@ -17,6 +22,7 @@
 #include "entities.h"
 #include "xml-error.h"
 #include "debugXML.h"
+#include "xmlIO.h"
 #include "SAX.h"
 
 /* #define DEBUG_SAX */
@@ -140,7 +146,7 @@ hasExternalSubset(void *ctx)
  * internalSubset:
  * @ctx: the user data (XML parser context)
  *
- * Does this document has an internal subset
+ * Callback on internal subset declaration.
  */
 void
 internalSubset(void *ctx, const xmlChar *name,
@@ -221,11 +227,11 @@ internalSubset(void *ctx, const xmlChar *name,
  * @publicId: The public ID of the entity
  * @systemId: The system ID of the entity
  *
- * Special entity resolver, better left to the parser, it has
- * more context than the application layer.
- * The default behaviour is to NOT resolve the entities, in that case
- * the ENTITY_REF nodes are built in the structure (and the parameter
- * values).
+ * The entity loader, to control the loading of external entities,
+ * the application can either:
+ *    - override this resolveEntity() callback in the SAX block
+ *    - or better use the xmlSetExternalEntityLoader() function to
+ *      set up it's own entity resolution routine
  *
  * Returns the xmlParserInputPtr if inlined or NULL for DOM behaviour.
  */
@@ -238,10 +244,8 @@ resolveEntity(void *ctx, const xmlChar *publicId, const xmlChar *systemId)
     fprintf(stderr, "SAX.resolveEntity(%s, %s)\n", publicId, systemId);
 #endif
 
-    if (systemId != NULL) {
-	return(xmlNewInputFromFile(ctxt, (char *) systemId));
-    }
-    return(NULL);
+    return(xmlLoadExternalEntity((const char *) systemId,
+				 (const char *) publicId, ctxt));
 }
 
 /**
@@ -1096,14 +1100,22 @@ void
 cdataBlock(void *ctx, const xmlChar *value, int len)
 {
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
-    xmlNodePtr ret;
+    xmlNodePtr ret, lastChild;
 
 #ifdef DEBUG_SAX
     fprintf(stderr, "SAX.pcdata(%.10s, %d)\n", value, len);
 #endif
-    ret = xmlNewCDataBlock(ctxt->myDoc, value, len);
-    xmlAddChild(ctxt->node, ret);
-    /* !!!!! merges */
+    lastChild = xmlGetLastChild(ctxt->node);
+#ifdef DEBUG_SAX_TREE
+    fprintf(stderr, "add chars to %s \n", ctxt->node->name);
+#endif
+    if ((lastChild != NULL) &&
+        (lastChild->type == XML_CDATA_SECTION_NODE)) {
+	xmlTextConcat(lastChild, value, len);
+    } else {
+	ret = xmlNewCDataBlock(ctxt->myDoc, value, len);
+	xmlAddChild(ctxt->node, ret);
+    }
 }
 
 /*
