@@ -14,10 +14,14 @@
 /* TODO add compression support, Send the Accept- , and decompress on the
         fly with ZLIB if found at compile-time */
 
-#ifndef WIN32
+#ifdef WIN32
+#define INCLUDE_WINSOCKS
+#include "win32config.h"
+#else
 #include "config.h"
 #endif
 
+
 #include <stdio.h>
 #include <string.h>
 
