@@ -8,8 +8,7 @@
  */
 
 #ifdef WIN32
-#define HAVE_FCNTL_H
-#include <io.h>
+#include "win32config.h"
 #else
 #include "config.h"
 #endif
@@ -26,6 +25,8 @@
 #include "parser.h"
 #include "parserInternals.h"
 
+/* TODO: use hash table for accesses to elem and attribute dedinitions */
+
 #define VERROR							\
    if ((ctxt != NULL) && (ctxt->error != NULL)) ctxt->error
 
@@ -495,6 +496,7 @@ xmlCopyElementTable(xmlElementTablePtr table) {
 	else
 	    cur->name = NULL;
 	cur->content = xmlCopyElementContent(ent->content);
+	/* TODO : rebuild the attribute list on the copy */
 	cur->attributes = NULL;
     }
     return(ret);
@@ -711,7 +713,7 @@ xmlScanAttributeDecl(xmlDtdPtr dtd, const xmlChar *elem) {
  * @ctxt:  the validation context
  * @elem:  the element name
  *
- * Veryfy that the element don't have too many ID attributes
+ * Verify that the element don't have too many ID attributes
  * declared.
  *
  * Returns the number of ID attributes found.
@@ -1502,7 +1504,7 @@ xmlFreeIDTable(xmlIDTablePtr table) {
 }
 
 /**
- * xmlIsID
+ * xmlIsID:
  * @doc:  the document
  * @elem:  the element carrying the attribute
  * @attr:  the attribute
@@ -1515,13 +1517,21 @@ xmlFreeIDTable(xmlIDTablePtr table) {
  */
 int
 xmlIsID(xmlDocPtr doc, xmlNodePtr elem, xmlAttrPtr attr) {
+    if (doc == NULL) return(0);
+    if (attr == NULL) return(0);
     if ((doc->intSubset == NULL) && (doc->extSubset == NULL)) {
         if (((attr->name[0] == 'I') || (attr->name[0] == 'i')) &&
             ((attr->name[1] == 'D') || (attr->name[1] == 'd')) &&
 	    (attr->name[2] == 0)) return(1);
+    } else if (doc->type == XML_HTML_DOCUMENT_NODE) {
+        if ((!xmlStrcmp(BAD_CAST "id", attr->name)) ||
+	    (!xmlStrcmp(BAD_CAST "name", attr->name)))
+	    return(1);
+	return(0);    
     } else {
 	xmlAttributePtr attrDecl;
 
+	if (elem == NULL) return(0);
 	attrDecl = xmlGetDtdAttrDesc(doc->intSubset, elem->name, attr->name);
 	if ((attrDecl == NULL) && (doc->extSubset != NULL))
 	    attrDecl = xmlGetDtdAttrDesc(doc->extSubset, elem->name,
@@ -1533,6 +1543,42 @@ xmlIsID(xmlDocPtr doc, xmlNodePtr elem, xmlAttrPtr attr) {
     return(0);
 }
 
+/**
+ * xmlRemoveID
+ * @doc:  the document
+ * @attr:  the attribute
+ *
+ * Remove the given attribute from the ID table maintained internally.
+ *
+ * Returns -1 if the lookup failed and 0 otherwise
+ */
+int
+xmlRemoveID(xmlDocPtr doc, xmlAttrPtr attr) {
+    xmlIDPtr cur;
+    xmlIDTablePtr table;
+    int i;
+
+    if (doc == NULL) return(-1);
+    if (attr == NULL) return(-1);
+    table = doc->ids;
+    if (table == NULL) 
+        return(-1);
+
+    /*
+     * Search the ID list.
+     */
+    for (i = 0;i < table->nb_ids;i++) {
+        cur = table->table[i];
+	if (cur->attr == attr) {
+	    table->nb_ids--;
+	    memmove(&table->table[i], &table->table[i+1],
+	            (table->nb_ids - i) * sizeof(xmlIDPtr));
+	    return(0);
+	}
+    }
+    return(-1);
+}
+
 /**
  * xmlGetID:
  * @doc:  pointer to the document
@@ -1721,7 +1767,7 @@ xmlFreeRefTable(xmlRefTablePtr table) {
 }
 
 /**
- * xmlIsRef
+ * xmlIsRef:
  * @doc:  the document
  * @elem:  the element carrying the attribute
  * @attr:  the attribute
@@ -1755,12 +1801,48 @@ xmlIsRef(xmlDocPtr doc, xmlNodePtr elem, xmlAttrPtr attr) {
     return(0);
 }
 
+/**
+ * xmlRemoveRef
+ * @doc:  the document
+ * @attr:  the attribute
+ *
+ * Remove the given attribute from the Ref table maintained internally.
+ *
+ * Returns -1 if the lookup failed and 0 otherwise
+ */
+int
+xmlRemoveRef(xmlDocPtr doc, xmlAttrPtr attr) {
+    xmlRefPtr cur;
+    xmlRefTablePtr table;
+    int i;
+
+    if (doc == NULL) return(-1);
+    if (attr == NULL) return(-1);
+    table = doc->refs;
+    if (table == NULL) 
+        return(-1);
+
+    /*
+     * Search the Ref list.
+     */
+    for (i = 0;i < table->nb_refs;i++) {
+        cur = table->table[i];
+	if (cur->attr == attr) {
+	    table->nb_refs--;
+	    memmove(&table->table[i], &table->table[i+1],
+	            (table->nb_refs - i) * sizeof(xmlRefPtr));
+	    return(0);
+	}
+    }
+    return(-1);
+}
+
 /**
  * xmlGetRef:
  * @doc:  pointer to the document
  * @Ref:  the Ref value
  *
- * Search the attribute declaring the given Ref
+ * Search the next attribute declaring the given Ref
  *
  * Returns NULL if not found, otherwise the xmlAttrPtr defining the Ref
  */
@@ -2724,6 +2806,7 @@ xmlValidateOneElement(xmlValidCtxtPtr ctxt, xmlDocPtr doc,
                       xmlNodePtr elem) {
     xmlElementPtr elemDecl;
     xmlElementContentPtr cont;
+    xmlAttributePtr attr;
     xmlNodePtr child;
     int ret = 1;
     const xmlChar *name;
@@ -2870,7 +2953,69 @@ xmlValidateOneElement(xmlValidCtxtPtr ctxt, xmlDocPtr doc,
 	    break;
     }
 
-    /* TODO - [ VC: Required Attribute ] */
+    /* [ VC: Required Attribute ] */
+    attr = elemDecl->attributes;
+    while (attr != NULL) {
+	if (attr->def == XML_ATTRIBUTE_REQUIRED) {
+	    xmlAttrPtr attrib;
+	    int qualified = -1;
+	    
+	    attrib = elem->properties;
+	    while (attrib != NULL) {
+		if (!xmlStrcmp(attrib->name, attr->name)) {
+		    if (attr->prefix != NULL) {
+		        xmlNsPtr nameSpace = attrib->ns;
+
+			if (nameSpace == NULL)
+			    nameSpace = elem->ns;
+			/*
+			 * qualified names handling is problematic, having a
+			 * different prefix should be possible but DTDs don't
+			 * allow to define the URI instead of the prefix :-(
+			 */
+			if (nameSpace == NULL) {
+			    if (qualified < 0) 
+				qualified = 0;
+	    		} else if (xmlStrcmp(nameSpace->prefix, attr->prefix)) {
+			    if (qualified < 1) 
+				qualified = 1;
+			} else
+			    goto found;
+		    } else {
+		        /*
+			 * We should allow applications to define namespaces
+			 * for their application even if the DTD doesn't 
+			 * carry one, otherwise, basically we would always
+			 * break.
+			 */
+			goto found;
+		    }
+		}
+		attrib = attrib->next;
+	    }
+	    if (qualified == -1) {
+		if (attr->prefix == NULL) {
+		    VERROR(ctxt->userData,
+		       "Element %s doesn't carry attribute %s\n",
+			   elem->name, attr->name);
+	        } else {
+		    VERROR(ctxt->userData,
+		       "Element %s doesn't carry attribute %s:%s\n",
+			   elem->name, attr->prefix,attr->name);
+		}
+	    } else if (qualified == 0) {
+		VWARNING(ctxt->userData,
+		   "Element %s required attribute %s:%s has no prefix\n",
+		       elem->name, attr->prefix,attr->name);
+	    } else if (qualified == 1) {
+		VWARNING(ctxt->userData,
+		   "Element %s required attribute %s:%s has different prefix\n",
+		       elem->name, attr->prefix,attr->name);
+	    }
+	}
+found:	    
+        attr = attr->next;
+    }
     return(ret);
 }
 
